
%if 0

UMBFILL - Allocates all free UMBs
Once written by someone, 2008 disassembled and partly rewritten by C. Masloch. Public Domain.
Updated 2022.

%endif


		cpu 8086
		org 256

start:
		xor ax, ax			; write a zero
		xchg ax, word [ 2Ch ]		; segment of program's environment block
		mov es, ax
		mov ah, 49h			; free memory
		int 21h
				; Note: You could argue that the environment block will be freed anyway
				;	because it isn't hacked to have another MCB owner. This is true,
				;	but also brings in a problem: If UMBFILL is loaded high (by means
				;	of such crap like LH), the environment block may also be loaded
				;	high. If it's then freed, UMBFILL didn't fill all UMBs.

		mov ax, 5802h			; get UMB link state
		int 21h
		jc error			; function not supported -->
		push ax				; save UMB link state

		mov ax, 5803h			; set UMB link state
		mov bx, 1			; enabled
		int 21h
		jc error_restore_link_state

		mov ax, 5800h			; get memory allocation strategy
		int 21h
		push ax				; save memory allocation strategy

		mov ax, 5801h			; set memory allocation strategy
		mov bx, 40h			; first fit, UMBs only
		int 21h

		mov dx, 0008h			; in case UMBFILL's own block is the only UMB (see below),
						;  set its MCB owner value to 0008h (owned by DOS)
		xor cx, cx			; set the UMB counter to zero

allocatenext:
		mov ah, 48h			; allocate memory
		mov bx, 0FFFFh			; requesting this amount will not succeed
		int 21h

		; test bx, bx			; largest block zero ?
		; jz allblocksdone		;  then no UMBs left
			; This logic was mistaken. There may be UMBs left that
			;  are only large enough to hold the MCB *itself*,
			;  that is the memory block's size is zero.
			; DOS allows to allocate those by passing a zero size
			;  to 21.48 so we should try to allocate in any case.
			; If we have exhausted the UMBs that call will fail.

		mov ah, 48h
		int 21h				; now do allocate the largest currently available block
		jc allblocksdone		; failed ? then no UMBs left -->
			; The value in bx is unchanged upon successful
			;  allocation, and holds the size of the memory block.

		inc cx				; count how many blocks

		mov dx, ax			; set dx for hackmcb and remember last allocated UMB in dx
		call hackmcb

		jmp short allocatenext


allblocksdone:
		mov ax, 5801h			; (re)set memory allocation strategy
		pop bx				; saved strategy
		int 21h

		mov ax, 5803h			; (re)set UMB link state
		pop bx				; saved UMB link state
		mov bh, 0			;  but only the low byte has meaning
		int 21h

		push cs
		pop ax
		cmp ax, 0A000h			; UMBFILL itself in an UMB ?
		jb checkifany			; no -->

		inc cx				; also count UMBFILL's UMB

		xor bx, bx			; indicate not to write new sig
		call hackmcb			; place the segment of the last UMB in UMBFILL's MCB owner field
					; (filling in another segment _is_ necessary because UMBFILL's own segment
					; is just what DOS will search for when freeing memory.)

checkifany:
		jcxz error			; no UMBs allocated at all ? -->

		mov dx, msg.done
		mov ah, 09h
		int 21h

		int 20h				; terminate (but UMBs with hacked MCBs will remain allocated)


error_restore_link_state:
		mov ax, 5803h			; (re)set UMB link state
		pop bx				; saved UMB link state
		mov bh, 0			;  but only the low byte has meaning
		int 21h

error:
		mov dx, msg.noumbs
		mov ah, 09h
		int 21h

		int 20h


	; Hack MCB to remain allocated
	;
	; INP:	ax => UMB, (ax - 1) => its MCB
	;	dx = value to fill into the MCB's owner field,
	;		mustn't equal cs (= UMBFILL's PSP)
	;	bx = flag, nonzero to write additional identification paragraph
	;		(usually the size of the memory block allocated,
	;		in paragraphs, except when called for our PSP block)
	; OUT:	-
	; CHG:	es, di, si
	; STT:	ds = ss = cs
hackmcb:
		dec ax				; ax pointed at the UMB; now MCB of that block
		mov es, ax			; => MCB
		inc ax				; restore ax
		 push cx
		mov cx, 4			; = number of words in MCB name
		mov di, 0008h			; -> MCB name field
		mov si, msg.mcbname		; place the string "UMBFILL",0 into the MCB 8-byte name field
						; (if not done, MEM shows random characters on some DOS versions)
		 push si
		rep movsw
		 pop si				; (preserve -> MCB name source)
		test bx, bx			; write the new ID paragraph ?
		jz .no_id_paragraph

			; es:di -> memory block start
		 push si
		mov cl, 4			; write 4 words again
		rep movsw
		 pop si				; (preserve -> MCB name source)
		mov cl, 4			; write 4 words again
		rep movsw

.no_id_paragraph:
		 pop cx
		mov word [ es:0001h ], dx	; place the segment of that block itself in its MCB owner field
						;  this prevents DOS from freeing the UMB when UMBFILL terminates
		retn


msg:
.noumbs:	db "No UMBs available",13,10,36
.done:		db "All UMBs allocated",13,10,36
	align 2, db 0
.mcbname:	db "UMBFILL"
		times 8 - ($ - .mcbname) db 0
