
%if 0

UMBFILLO - Allocates all free UMBs (old version)
Once written by someone, 2008 disassembled and partly rewritten by C. Masloch. Public Domain.

Assemble with NASM 2.03.01: NASM.EXE -fbin -oUMBFILLO.COM UMBFILLO.ASM
 (Some other NASM versions should also work)

Problems of UMBFILLO:
- If UMBFILL's PSP itself was loaded high, it's (partial) freeing makes one remaining free UMB.
- Environment segment left in (low) memory.
- Some DOS versions will not completely free the PSP segment if Int27 is executed with dx=0.
- Due to used pusha and popa opcodes, UMBFILL may not work as expected on 8086 processors.

Technical issues with UMBFILLO:
- Data at start of binary required jump to actual start of code.
- Some totally useless code at the start of the file that left out the actual int opcode,
  this also caused UMBFILL to always report an error if the carry flag was set on entry.
- Some doubled conditional code (the second code is only reached if the condition is true,
  making the conditional jump opcode always jump).
- Some data that's not required, using the stack is more efficient.
- Some data that's updated, but never used for anything.

Use my UMBFILL instead. It fixes all above problems and even comes with another program that
will free the allocated UMBs again.

%endif


		cpu 186
		org 100h

start:
		jmp codestart

largest:	dw 0				; size of last allocated UMB
umblink:	db 0				; saved UMB link state
allocstrat:	dw 0				; saved memory allocation strategy
allocatedblocks:db 0				; number of UMBs allocated yet

msg:
.noumbs:	db "No UMBs availlable",13,10,36
.done:		db "All UMBs allocated",13,10,36

disp_msg:
		mov ah, 09h
		int 21h
		retn

codestart:
		pusha

		push cs
		pop ds

		mov ax, 5802h
		mov bx, 0
		jc error			; ??? stupid

		mov ax, 5802h			; get UMB link state
		mov bx, 0
		int 21h
		mov byte [ umblink ], al	; save UMB link state

		mov ax, 5800h			; get memory allocation strategy
		int 21h
		mov word [ allocstrat ], ax	; save memory allocation strategy

		mov ax, 5803h			; set UMB link state
		mov bx, 1			; enabled
		int 21h

		mov ax, 5801h			; set memory allocation strategy
		mov bx, 40h			; first fit, UMBs only
		int 21h

allocatenext:
		mov ax, 4800h			; allocate memory
		mov bx, 0FFFFh			; requesting this amount will not succeed
		int 21h
		mov word [ largest ], bx	; but it returns the largest currently available block

		cmp bx, byte 0			; largest block zero ?
		je allblocksdone		;  then no UMBs left

		inc byte [ allocatedblocks ]	; count how many blocks

		mov ax, 4800h
		int 21h				; now do allocate the largest currently available block
						; (not expected to return with error)

		cmp word [ largest ], byte 0
		jnz allocatenext		; (the condition is always true)

allblocksdone:
		mov ax, 5801h			; (re)set memory allocation strategy
		mov bx, word [ allocstrat ]	; saved strategy
		int 21h

		mov ax, 5803h			; (re)set UMB link state
		mov bh, 0
		mov bl, [ umblink ]		; saved UMB link state
		int 21h

		cmp byte [ allocatedblocks ], 0	; no UMBs allocated at all ?
		je error			; report -->

		mov dx, msg.done
		call disp_msg

		popa

		mov dx, 0			; number of bytes of program's block to hold resident
		int 27h

error:
		mov dx, msg.noumbs
		call disp_msg

		popa

		int 20h
